import 'dart:async';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/foundation.dart';
import 'package:wowit/common.dart';
import 'package:wowit/interfaces/auth/auth.dart';
import 'package:wowit/interfaces/auth/mixins/fake_login_mixin.dart';
import 'package:wowit/interfaces/auth/mixins/firebase_login_mixin.dart';
import 'package:wowit/interfaces/auth/mixins/sql_login_mixin.dart';
import 'package:wowit/methods.dart';
import 'package:get/get.dart';
import 'package:wowit/interfaces/mixins/handle_exception_mixin.dart';
import 'package:wowit/message.dart';

mixin FirebaseOTPMixin on FirebaseLoginMixin, BaseAuth {
  String? _verificationId;

  Future verifyPhoneNumber(
    String phone, {
    required Function verifyOTP,
  }) async {
    final String phoneNumber = Methods.preparePhone(phone);
    Common.phoneNumberInput = phoneNumber;
    if (this is FakeLoginMixin) {
      if (phoneNumber == (this as FakeLoginMixin).fakePhone) {
        _verificationId = '123456';
        verifyOTP();
        return null;
      }
    }
    await FirebaseAuth.instance.verifyPhoneNumber(
      phoneNumber: phoneNumber,
      timeout: const Duration(seconds: 0),
      verificationCompleted: (authCredential) {},
      verificationFailed: (e) {
        _verificationId = null;
        handelException(e, subTitle: 'verifyPhoneNumber');
      },
      codeSent: (verificationId, _) {
        _verificationId = verificationId;
        verifyOTP();
      },
      codeAutoRetrievalTimeout: (verificationId) {
        _verificationId = verificationId;
      },
    );
  }

  Future<bool> singInWithOTP(
    String otp, {
    Function(dynamic e)? onError,
  }) async {
    if (_verificationId == null) return false;
    if (this is FakeLoginMixin) {
      if (otp == '123456' &&
          Common.phoneNumberInput == (this as FakeLoginMixin).fakePhone) {
        if (kDebugMode) {
          print('================');
          print('|| fake login ||');
          print('================');
        }
        return true;
      }
    }
    var _credential = PhoneAuthProvider.credential(
      verificationId: _verificationId!,
      smsCode: otp,
    );
    try {
      await FirebaseAuth.instance.signInWithCredential(_credential);
      User? user = FirebaseAuth.instance.currentUser;
      if (user != null) {
        await initAccountFirebase(phone: user.phoneNumber!, uidPhone: user.uid);
        if (this is SQLLoginMixin) {
          await (this as SQLLoginMixin)
              .initAccountSQL(phone: user.phoneNumber!, uidPhone: user.uid);
        }
        return true;
      }
      return false;
    } catch (e) {
      if (onError != null) {
        onError(e);
      } else {
        handelException(e, subTitle: 'singInWithOTP');
      }
      return false;
    }
  }
}

mixin HandleExceptionOTPMixin on HandleExceptionMixin, FirebaseOTPMixin {
  void handleExceptionOTP(dynamic e) {
    String error = e.toString();
    if (error.contains('code-expired')) {
      theSMSCodeHasExpried();
    } else if (error.contains('invalid-verification-code')) {
      theSMSOTPInValid();
    } else if (error.contains('missing-verification-code')) {
      theSMSOTPMissing();
    } else if (error.contains('invalid-phone-number')) {
      invalidPhone();
    } else {
      if (Common.supperAdmin || kDebugMode) {
        Get.snackbar(
          Message.getMessage('REGISTER_THONG_BAO'),
          e.toString(),
          snackPosition: SnackPosition.TOP,
          duration: const Duration(milliseconds: 10000),
        );
      }
    }
  }

  void theSMSOTPInValid() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_OPT_IN_VALID'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(milliseconds: 10000),
    );
  }

  void theSMSCodeHasExpried() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_HAS_EXPIRED'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(milliseconds: 10000),
    );
  }

  void theSMSOTPMissing() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_HAS_MISSING'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(milliseconds: 10000),
    );
  }

  void invalidPhone() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_INVALID_PHONE'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(milliseconds: 10000),
    );
  }
}
